package com.guozhi.elasticsearch;

import com.alibaba.fastjson.JSON;
import com.guozhi.elasticsearch.mapper.LogMapper;
import lombok.extern.slf4j.Slf4j;
import org.frameworkset.elasticsearch.ElasticSearchHelper;
import org.frameworkset.tran.DataStream;
import org.frameworkset.tran.db.input.es.DB2ESImportBuilder;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import tk.mybatis.spring.annotation.MapperScan;

import javax.annotation.Resource;

/**
 * 异步导入 （单次）
 * @author LiuchangLan
 * @date 2020/10/21 16:26
 */
@SpringBootTest
@Slf4j
@MapperScan(value = "com.guozhi.elasticsearch.mapper")
public class EsImport {

    @Resource
    private LogMapper logMapper;

    @Test
    void importData(){
        // 获取批量同步对象
        DB2ESImportBuilder importBuilder = DB2ESImportBuilder.newInstance();

        //清除测试表
        ElasticSearchHelper.getRestClientUtil().dropIndice("archives");

        //数据源相关配置，可选项，可以在外部启动数据源
        importBuilder.setDbName("archives") //库命
                .setDbDriver("com.mysql.cj.jdbc.Driver") //数据库驱动程序，必须导入相关数据库的驱动jar包
                .setDbUrl("jdbc:mysql://115.159.38.225:3306/archives?autoReconnect=true&useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=CONVERT_TO_NULL&useSSL=false&serverTimezone=GMT%2B8")//通过useCursorFetch=true启用mysql的游标fetch机制，否则会有严重的性能隐患，useCursorFetch必须和jdbcFetchSize参数配合使用，否则不会生效
                .setDbUser("root")
                .setDbPassword("abcd123456!@#")
                .setValidateSQL("select 1") //检查连接sql
                .setUsePool(false);//是否使用连接池
        //指定导入数据的sql语句，必填项，可以设置自己的提取逻辑
        importBuilder.setSql("select * from sp_log");

        importBuilder
                .setIndex("archives") //必填项 索引名
                .setIndexType("archives") //es 7以后的版本不需要设置indexType，es7以前的版本必需设置indexType
                .setRefreshOption(null)//可选项，null表示不实时刷新，importBuilder.setRefreshOption("refresh");表示实时刷新
                .setUseJavaName(true) //可选项,将数据库字段名称转换为java驼峰规范的名称，例如:doc_id -> docId
                .setBatchSize(5000)  //可选项,批量导入es的记录数，默认为-1，逐条处理，> 0时批量处理
                .setJdbcFetchSize(10000);//设置数据库的查询fetchsize，同时在mysql url上设置useCursorFetch=true启用mysql的游标fetch机制，否则会有严重的性能隐患，jdbcFetchSize必须和useCursorFetch参数配合使用，否则不会生效

        /**
         * 一次、作业创建一个内置的线程池，实现多线程并行数据导入elasticsearch功能，作业完毕后关闭线程池
         */
        importBuilder.setParallel(true);//设置为多线程并行批量导入
        importBuilder.setQueue(100);//设置批量导入线程池等待队列长度
        importBuilder.setThreadCount(200);//设置批量导入线程池工作线程数量
        importBuilder.setContinueOnError(true);//任务出现异常，是否继续执行作业：true（默认值）继续执行 false 中断作业执行
        importBuilder.setAsyn(false);//true 异步方式执行，不等待所有导入作业任务结束，方法快速返回；false（默认值） 同步方式执行，等待所有导入作业任务结束，所有作业结束后方法才返回
        importBuilder.setRefreshOption("refresh"); // 为了实时验证数据导入的效果，强制刷新数据，生产环境请设置为null或者不指定

        /**
         * 执行数据库表数据导入es操作
         */
        DataStream dataStream = importBuilder.builder();
        dataStream.execute();
        dataStream.destroy();//执行完毕后释放资源
        //获取索引表dbdemo中的数据总量，如果没有设置refreshOption,es插入数据会有几秒的延迟（具体的延迟多久取决于index refresh interval配置），所以countAll统计出来的结果不一定准确
        long count = ElasticSearchHelper.getRestClientUtil().countAll("archives");
        System.out.println("数据导入完毕后索引表dbdemo中的文档数量:"+count);
    }
}
